==> fishsupper-0.1.3.tar.gz: fishsupper-0.1.3/src/gamemanager.cc <==
//  Fish Supper
//  Copyright (C) 2006 Matthew Clarke <matthew_larcke@lycos.co.uk>
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.




#include "gamemanager.h"
#include <iostream>
#include <cstdlib>




using std::cout;
using std::cerr;
using std::endl;




//extern bool joystick_available;




// *******************
// *** CONSTRUCTOR ***
// *******************
FS::GameManager::GameManager(SDL_Surface * s) : ts_display(s),
                                                gr_display(s),
                                                p_display(s),
                                                s_display(s, &my_settings)                                          
{
    screen = s;
    
    current_state = INTRO;
    
    p_model.init_sprite_manager(&p_display, &snd_engine);
    
    snd_engine.turn_on( my_settings.is_sound() );
    
} // FS::GameManager::GameManager()

// ******************
// *** DESTRUCTOR ***
// ******************
FS::GameManager::~GameManager()
{
    // does nothing yet...
    
} // FS::GameManager::~GameManager()




// ************************
// *** MEMBER FUNCTIONS ***
// ************************

// **************************************************

void FS::GameManager::start_loop()
{
    SDL_Event event;
    my_key_event = NONE;
    loop_running = true;
    
    key_down = false;
    
    // Before we enter the loop, display the title screen
    ts_display.redraw();
        
    while (loop_running)
    {
        while ( SDL_PollEvent(&event) )
        {
            switch (event.type)
            {
                case SDL_KEYDOWN:
                case SDL_KEYUP:
                case SDL_JOYAXISMOTION:
                case SDL_JOYBUTTONDOWN:
                    analyse_event(&event);
                    break;
                case SDL_QUIT:
                    loop_running = false;
                    break;        
            } // switch
        } // while
        
        switch (current_state)
        {
            case INTRO:
                if ( my_key_event == ESCAPE_PRESSED )
                {
                    loop_running = false;
                }
                else
                {
                    handle_selection( ts_display.update(my_key_event) );
                } // if ... else
                break;
                
            case GET_READY:
                gr_display.redraw(timer.get_time(), my_key_event);
                if (gr_display.is_finished())
                {
                    // start play display
                    current_state = PLAY;
                    p_model.new_level(timer.get_time());
                    p_display.reset();
                } // if
                break;
                
            case PLAY:
                if (my_key_event == PAUSE_KEY_PRESSED)
                {
                    timer.pause();
                } // if
                
                if (timer.is_running())
                {
                    switch ( p_model.update(timer.get_time(), my_key_event) )
                    {
                        case PR_OK:
                            p_display.redraw();
                            break;
                        case PR_LEVEL_COMPLETED:
                            current_state = GET_READY;
                            gr_display.start_draw( timer.get_time(), p_model.get_level() );
                            break;
                        case PR_GAME_OVER:
                            if ( hs_display.check_candidate(p_model.get_score()) )
                            {
                                current_state = ENTERING_NAME;
                                hs_display.init_input(p_model.get_score(), screen);
                            }
                            else
                            {
                                current_state = INTRO;
                                ts_display.redraw();
                            } // if ... else
                            break;
                        case PR_GAME_ABORTED:
                            current_state = INTRO;
                            ts_display.redraw();
                            break;
                    } // switch
                } // if
                break;
                
            case SETTINGS:
                if ( my_key_event == ESCAPE_PRESSED || s_display.redraw(my_key_event) )
                {
                    snd_engine.turn_on( my_settings.is_sound() );
                    current_state = INTRO;
                    ts_display.redraw();
                } // if        
                break;
                
            case HI_SCORES:
                if (my_key_event == ESCAPE_PRESSED || my_key_event == JUMP_KEY_PRESSED)
                {
                    current_state = INTRO;
                    ts_display.redraw();
                } // if    
                break;
                
            case ENTERING_NAME:
                if (key_down)
                {
                    if ( hs_display.input(event.key.keysym, screen) )
                    {
                        current_state = HI_SCORES;
                        hs_display.show_high_scores(screen);
                    } // if
                } // if
                break;
                
        } // switch
        
        SDL_Delay(10);  // 100 fps
        
        // IMPORTANT!!! Clear my_key_event!
        my_key_event = NONE;
        key_down = false;
        
    } // while

} // FS::GameManager::start_loop()

// **************************************************

void FS::GameManager::analyse_event(const SDL_Event * event)
{
    bool joystick_selected = my_settings.is_joystick_selected();
    
    if (event->type == SDL_KEYDOWN)
    {
        key_down = true;
        
        if (event->key.keysym.sym == SDLK_ESCAPE)
        {
            my_key_event = ESCAPE_PRESSED;
        }
        else if (event->key.keysym.sym == SDLK_p)
        {
            my_key_event = PAUSE_KEY_PRESSED;
        } 
        else if ( !joystick_selected )
        {
            if (event->key.keysym.sym == SDLK_UP)
            {
                my_key_event = UP_KEY_PRESSED;
            }
            else if (event->key.keysym.sym == SDLK_RIGHT)
            {
                my_key_event = RIGHT_KEY_PRESSED;      
            }
            else if (event->key.keysym.sym == SDLK_DOWN)
            {
                my_key_event = DOWN_KEY_PRESSED;
            }
            else if (event->key.keysym.sym == SDLK_LEFT)
            {
                my_key_event = LEFT_KEY_PRESSED;
            }
            else if (event->key.keysym.sym == SDLK_SPACE)
            {
                my_key_event = JUMP_KEY_PRESSED;
            } // if ... else
        } // if ... else
    }
    else if ( (!joystick_selected) && (event->type == SDL_KEYUP) )
    {
        if (event->key.keysym.sym == SDLK_UP)
        {
            my_key_event = UP_KEY_RELEASED;
        }
        else if (event->key.keysym.sym == SDLK_RIGHT)
        {
            my_key_event = RIGHT_KEY_RELEASED;
        }
        else if (event->key.keysym.sym == SDLK_DOWN)
        {
            my_key_event = DOWN_KEY_RELEASED;
        }
        else if (event->key.keysym.sym == SDLK_LEFT)
        {
            my_key_event = LEFT_KEY_RELEASED;
        } // if ... else
    } 
    else if ( joystick_selected && (event->type == SDL_JOYAXISMOTION) )
    {
        if ( event->jaxis.axis == 0 ) // left/right
        {
            if ( event->jaxis.value < -3200 )
            {
                my_key_event = last_horizontal = LEFT_KEY_PRESSED;
            }
            else if ( !event->jaxis.value )
            {
                my_key_event = (last_horizontal == LEFT_KEY_PRESSED) ? LEFT_KEY_RELEASED :
                        RIGHT_KEY_RELEASED;
            }
            else if ( event->jaxis.value > 3200 ) 
            {
                my_key_event = last_horizontal = RIGHT_KEY_PRESSED;
            } // if ... else
        }
        else if ( event->jaxis.axis == 1 ) 
        {
            if ( event->jaxis.value < -3200 )
            {
                my_key_event = last_vertical = UP_KEY_PRESSED;
            }
            else if ( !event->jaxis.value )
            {
                my_key_event = (last_vertical == UP_KEY_PRESSED) ? UP_KEY_RELEASED :
                        DOWN_KEY_RELEASED;
            }
            else if ( event->jaxis.value > 3200 )
            {
                my_key_event = last_vertical = DOWN_KEY_PRESSED;
            } // if ... else
        } // if ... else
    }
    else if (joystick_selected && event->type == SDL_JOYBUTTONDOWN)
    {
        my_key_event = JUMP_KEY_PRESSED;
    } // if ... else
    
} // FS::GameManager::analyse_event() 

// **************************************************

void FS::GameManager::handle_selection(TitleScreenDisplay::Selection sel)
{
    switch (sel)
    {
        case TitleScreenDisplay::NONE:
            break;
    	case TitleScreenDisplay::PLAY:
            current_state = GET_READY;
            p_model.new_game();
            gr_display.start_draw( timer.get_time(), p_model.get_level() );
            break;
        case TitleScreenDisplay::SETTINGS:
            current_state = SETTINGS;
            s_display.start_draw();
            break;
        case TitleScreenDisplay::HI_SCORES:
            current_state = HI_SCORES;
            hs_display.show_high_scores(screen);
            break;
        case TitleScreenDisplay::QUIT:
            loop_running = false;
            break;
    } // switch
    
} // FS::GameManager::handle_selection()
 
// **************************************************
// **************************************************
// **************************************************
// **************************************************
// **************************************************

==> fishsupper-0.1.4.tar.gz: fishsupper-0.1.4/src/gamemanager.cc <==
// $Id: gamemanager.cc,v 1.4 2006/08/05 09:08:58 matthew Exp $

//  Fish Supper
//  Copyright (C) 2006 Matthew Clarke <matthew_larcke@lycos.co.uk>
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.




#include "gamemanager.h"
#include <iostream>
#include <cstdlib>




// *******************
// *** CONSTRUCTOR ***
// *******************
FS::GameManager::GameManager(SDL_Surface * s) : ts_display(s),
                                                gr_display(s),
                                                p_display(s),
                                                s_display(s, &my_settings)                                          
{
    screen = s;
    
    current_state = INTRO;
    
    p_model.init_sprite_manager(&p_display, &snd_engine);
    
    snd_engine.turn_on( my_settings.is_sound() );
    
} // FS::GameManager::GameManager()

// ******************
// *** DESTRUCTOR ***
// ******************
FS::GameManager::~GameManager()
{
    // does nothing yet...
    
} // FS::GameManager::~GameManager()




// ************************
// *** MEMBER FUNCTIONS ***
// ************************

// **************************************************

void FS::GameManager::start_loop()
{
    SDL_Event event;
    my_key_event = NONE;
    loop_running = true;
    int i;  // used in some of 'while' loops below
    key_down = false;
    
    // Before we enter the loop, display the title screen
    ts_display.redraw();
        
    while (loop_running)
    {
        num_key_events = num_key_downs = 0;
        while ( SDL_PollEvent(&event) )
        {
            switch (event.type)
            {
                case SDL_KEYDOWN:
                    if ( current_state == ENTERING_NAME )
                    {
                        my_key_downs[num_key_downs++] = event.key.keysym;
                        break;
                    } // if
                    // allow 'fall-through'
                case SDL_KEYUP:
                case SDL_JOYAXISMOTION:
                case SDL_JOYBUTTONDOWN:
                    analyse_event(&event);
                    break;
                case SDL_QUIT:
                    loop_running = false;
                    break;        
            } // switch
        } // while
        
        switch (current_state)
        {
            case INTRO:
                if ( num_key_events )
                {
                    handle_selection( ts_display.update(my_key_events, num_key_events) );
                } // if
                break;
                
            case GET_READY:
                if ( gr_display.redraw(timer.get_time(), my_key_events, num_key_events) )
                {
                    // start play display
                    current_state = PLAY;
                    p_model.new_level(timer.get_time());
                    p_display.reset();
                } // if
                break;
                
            case PLAY:
                i = 0;
                while ( i < num_key_events )
                {
                    if ( my_key_events[i] == ESCAPE_PRESSED )
                    {
                        snd_engine.stop_all_sounds();
                        current_state = INTRO;
                        ts_display.redraw();
                        // make sure timer is running
                        timer.resume();
                        break;
                    }
                    else if ( my_key_events[i] == PAUSE_KEY_PRESSED )
                    {
                        if (timer.pause())
                        {
                            snd_engine.resume_all_sounds();
                        }
                        else
                        {
                            snd_engine.pause_all_sounds();
                        } // if ... else
                        break;
                    } // if ... else
                    ++i;
                } // while
                
                if ( timer.is_running() && current_state == PLAY )        
                {
                    switch ( p_model.update(timer.get_time(), my_key_events, num_key_events) )
                    {
                        case PR_OK:
                            p_display.redraw();
                            break;
                        case PR_LEVEL_COMPLETED:
                            current_state = GET_READY;
                            gr_display.start_draw( timer.get_time(), p_model.get_level() );
                            break;
                        case PR_GAME_OVER:
                            if ( hs_display.check_candidate(p_model.get_score()) )
                            {
                                current_state = ENTERING_NAME;
                                hs_display.init_input(p_model.get_score(), screen);
                            }
                            else
                            {
                                current_state = INTRO;
                                ts_display.redraw();
                            } // if ... else
                            break;
                    } // switch
                } // if
                break;
                
            case SETTINGS:
                // No point calling redraw() if there was no key event...
                if ( num_key_events && s_display.redraw(my_key_events, num_key_events) )
                {
                    snd_engine.turn_on( my_settings.is_sound() );
                    current_state = INTRO;
                    ts_display.redraw();
                } // if        
                break;
                
            case HI_SCORES:
                i = 0;
                while (i < num_key_events)
                {
                    if ( my_key_events[i] == ESCAPE_PRESSED ||
                            my_key_events[i] == JUMP_KEY_PRESSED )
                    {
                        current_state = INTRO;
                        ts_display.redraw();
                    } // if
                    ++i;
                } // while    
                break;
                
            case ENTERING_NAME:
                if ( num_key_downs && hs_display.input(my_key_downs, num_key_downs, screen) )
                {
                    current_state = HI_SCORES;
                    hs_display.show_high_scores(screen);
                } // if
                /*
                if (key_down)
                {
                    if ( hs_display.input(event.key.keysym, screen) )
                    {
                        current_state = HI_SCORES;
                        hs_display.show_high_scores(screen);
                    } // if
                } // if
                */
                break;
                
        } // switch
        
        SDL_Delay(10);  // 100 fps
        
        // IMPORTANT!!! Clear my_key_event!
        my_key_event = NONE;
        key_down = false;
        
    } // while

    my_settings.write_config_file();
    
} // FS::GameManager::start_loop()

// **************************************************

void FS::GameManager::analyse_event(const SDL_Event * event)
{
    bool joystick_selected = my_settings.is_joystick_selected();
    
    if (event->type == SDL_KEYDOWN)
    {
        if (event->key.keysym.sym == SDLK_ESCAPE)
        {
            my_key_event = ESCAPE_PRESSED;
        }
        else if (event->key.keysym.sym == SDLK_p)
        {
            my_key_event = PAUSE_KEY_PRESSED;
        } 
        else if ( !joystick_selected )
        {
            if (event->key.keysym.sym == SDLK_UP)
            {
                my_key_event = UP_KEY_PRESSED;
            }
            else if (event->key.keysym.sym == SDLK_RIGHT)
            {
                my_key_event = RIGHT_KEY_PRESSED;      
            }
            else if (event->key.keysym.sym == SDLK_DOWN)
            {
                my_key_event = DOWN_KEY_PRESSED;
            }
            else if (event->key.keysym.sym == SDLK_LEFT)
            {
                my_key_event = LEFT_KEY_PRESSED;
            }
            else if (event->key.keysym.sym == SDLK_SPACE)
            {
                my_key_event = JUMP_KEY_PRESSED;
            } // if ... else
        } // if ... else
    }
    else if ( (!joystick_selected) && (event->type == SDL_KEYUP) )
    {
        if (event->key.keysym.sym == SDLK_UP)
        {
            my_key_event = UP_KEY_RELEASED;
        }
        else if (event->key.keysym.sym == SDLK_RIGHT)
        {
            my_key_event = RIGHT_KEY_RELEASED;
        }
        else if (event->key.keysym.sym == SDLK_DOWN)
        {
            my_key_event = DOWN_KEY_RELEASED;
        }
        else if (event->key.keysym.sym == SDLK_LEFT)
        {
            my_key_event = LEFT_KEY_RELEASED;
        } // if ... else
    } 
    else if ( joystick_selected && (event->type == SDL_JOYAXISMOTION) )
    {
        if ( event->jaxis.axis == 0 ) // left/right
        {
            if ( event->jaxis.value < -3200 )
            {
                my_key_event = last_horizontal = LEFT_KEY_PRESSED;
            }
            else if ( !event->jaxis.value )
            {
                my_key_event = (last_horizontal == LEFT_KEY_PRESSED) ? LEFT_KEY_RELEASED :
                        RIGHT_KEY_RELEASED;
            }
            else if ( event->jaxis.value > 3200 ) 
            {
                my_key_event = last_horizontal = RIGHT_KEY_PRESSED;
            } // if ... else
        }
        else if ( event->jaxis.axis == 1 ) 
        {
            if ( event->jaxis.value < -3200 )
            {
                my_key_event = last_vertical = UP_KEY_PRESSED;
            }
            else if ( !event->jaxis.value )
            {
                my_key_event = (last_vertical == UP_KEY_PRESSED) ? UP_KEY_RELEASED :
                        DOWN_KEY_RELEASED;
            }
            else if ( event->jaxis.value > 3200 )
            {
                my_key_event = last_vertical = DOWN_KEY_PRESSED;
            } // if ... else
        } // if ... else
    }
    else if (joystick_selected && event->type == SDL_JOYBUTTONDOWN)
    {
        my_key_event = JUMP_KEY_PRESSED;
    } // if ... else
    
    my_key_events[num_key_events++] = my_key_event;
    
} // FS::GameManager::analyse_event() 

// **************************************************

void FS::GameManager::handle_selection(TitleScreenDisplay::Selection sel)
{
    switch (sel)
    {
        case TitleScreenDisplay::NONE:
            break;
    	case TitleScreenDisplay::PLAY:
            current_state = GET_READY;
            p_model.new_game();
            gr_display.start_draw( timer.get_time(), p_model.get_level() );
            break;
        case TitleScreenDisplay::SETTINGS:
            current_state = SETTINGS;
            s_display.start_draw();
            break;
        case TitleScreenDisplay::HI_SCORES:
            current_state = HI_SCORES;
            hs_display.show_high_scores(screen);
            break;
        case TitleScreenDisplay::QUIT:
            loop_running = false;
            break;
    } // switch
    
} // FS::GameManager::handle_selection()
 
// **************************************************
// **************************************************
// **************************************************
// **************************************************
// **************************************************
